/* 
 * Copyright (c)2007-2008, Dustin Spicuzza
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of the Dustin Spicuzza may not be used to endorse or promote 
 *       products derived from this software without specific prior written 
 *       permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Dustin Spicuzza ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Dustin Spicuzza BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using System.Threading;

namespace Panes {
	
	/// <summary>
	/// This is the root management control for the Pane interface
	/// </summary>
	public partial class PaneContainer : PaneBase {

		int removing = 0;

		#region Constructors

		public PaneContainer(bool isRoot) {

			this.IsRoot = isRoot;
			//this.Dock = DockStyle.Fill;
		}

		public PaneContainer() {
			//this.Dock = DockStyle.Fill;
		}

		#endregion

		#region Overrides

		protected override void OnControlRemoved(ControlEventArgs e) {
			base.OnControlRemoved(e);

			if ( removing == 0) {

				if ( (this.Parent != null && this.Parent.GetType() == typeof(FloatingPane)) || !IsRoot ){

					if ( this.Controls.Count <= 0 ) {
						this.Parent = null;
						this.Dispose();

					}
				}
			}
		}

		#endregion

		/// <summary>
		/// This is called after a control or controls are removed from the container,
		/// and tries to decide the best way to handle it.
		/// </summary>
		internal void DoReorganize() {

			// should already be handled somewhere else
			if ( IsDisposed || this.Controls.Count <= 0)
				return;

			if ( ( this.Parent != null && this.Parent.GetType() == typeof(FloatingPane)) || !IsRoot ) {

				if ( this.Controls.Count == 1 ) {

					Interlocked.Exchange(ref removing, 1);

					Type t = this.Controls[0].GetType();

					// if it is a panebase, then replace self with it
					if ( t.BaseType == typeof(PaneBase)) {

						PaneBase pb = Controls[0] as PaneBase;
						
						// set these properties correctly, since its replacing us
						pb.mySplitter = mySplitter;
						pb.IsRoot = IsRoot;
						pb.Dock = Dock;
						pb.Size = Size;

						mySplitter = null;

						int idx = Parent.Controls.IndexOf(this);

						// remove from self, and add to parent
						pb.Parent = null;

						Control parent = Parent;

						Parent.SuspendLayout();
						Parent.Controls.Add(pb);
						Parent.Controls.SetChildIndex(pb, idx);
						
						// remove self from parent and dispose
						Parent = null;
						parent.ResumeLayout();
						Dispose();
					}

					Interlocked.Exchange(ref removing, 0);

				} else if ( this.Controls.Count == 2 ) {
					
					// if there is a control that has a fill style already, then do nothing -- since
					// the result was what the user expected to see. However, if there is not one 
					// with a fill style, then we need to decide which one needs to fill
					bool foundFill = false;
					foreach (Control c in Controls){
						if ( c.Dock == DockStyle.Fill ) {
							foundFill = true;
							break;
						}
					}

					if (!foundFill){
						// the first one will always be a splitter, so remove it
						this.SuspendLayout();
						Controls.RemoveAt(0);
						Controls[0].Dock = DockStyle.Fill;

						// call recursively, isn't infinite since there is a fill member now
						DoReorganize();

						if ( !this.IsDisposed )
							this.ResumeLayout();
					}
				}
			}


		}

		/// <summary>
		/// Add a pane to the container
		/// </summary>
		/// <param name="p">The pane to add</param>
		/// <param name="position">What position it should take up</param>
		public override void AddPane(PaneBase pb, DockStyle position) {
	
			this.SuspendLayout();
			
			// this is the primary difference: filling the container is always easy, don't 
			// need to worry about kids since there shouldn't be any
			if ( position == DockStyle.Fill ) {
				
				// should provide a mechanism for windows to be tabbed or something to
				// that effect.

				// if (frontcontrol dockstyle is fill)
				//		if its not a tabbed thing
				//			make it a tabbed thing
				//		add to tab

				// so, if there is already a fill control...
				if ( Controls.Count > 0 && Controls[0].Dock == DockStyle.Fill ) {

					if ( Controls[0].GetType() == typeof(TabPane) ) {

						// add item
						( Controls[0] as TabPane ).AddPane(pb, DockStyle.Fill);

					} else {

						// create new tabpane
						TabPane tp = new TabPane(TabPane.TabPaneStyle.Bottom);
						PaneBase opb = Controls[0] as PaneBase;
						Controls.Remove(opb);

						// set it up in ourself
						Controls.Add(tp);
						tp.Dock = DockStyle.Fill;
						tp.BringToFront();

						// add item
						tp.AddPane(opb, DockStyle.Fill);
						tp.AddPane(pb, DockStyle.Fill);
					}


				} else {
					Controls.Add(pb);
					pb.Dock = DockStyle.Fill;
					pb.BringToFront();
				}

			} else {

				// add a splitter, associate it with the pane
				Splitter s = new Splitter();

				pb.mySplitter = s;

				s.Dock = pb.Dock = position;

				Controls.Add(s);
				Controls.Add(pb);

				// finally, ensure each component has equal size
				if ( position == DockStyle.Left || position == DockStyle.Right ) {
					if ( pb.Width < Width / 4 )
						pb.Width = Width / 4;
				} else {
					if (pb.Height < Height / 4)
						pb.Height = Height / 4;
				}
			}

			this.ResumeLayout();
			pb.IsRoot = false;
		}

		/// <summary>
		/// This function is called during a mouse mouse of a floating pane, to determine whether or not
		/// to show the drag edges or not. This will try to call all of its children as well. 
		/// </summary>
		/// <param name="mousePosition">Point to check</param>
		/// <param name="hasMouse">This is set to true if you might have the mouse positioned inside of you. If it
		/// is set to false, then theres no way the mouse is inside of you.</param>
		/// <returns>Returns either tryMouse, or false if it displayed a drag window</returns>
		internal override bool UpdateDragWindow(Point mousePosition, bool tryMouse) {

			// if we can't display anything, then return.. it doesn't matter
			if ( !canDisplayDragEdges )
				return tryMouse;

			// this doesn't use the base implementation
			bool hasMouse = false;

			// check yourself first
			if ( tryMouse ) {

				if (RectangleToScreen(ClientRectangle).Contains(mousePosition) ) {

					hasMouse = true;
					tryMouse = false;

					// always show if you're root and not floating -- otherwise never show
					if ( IsRoot && this.Parent.GetType() != typeof(FloatingPane))
						positionControls.UpdateDragWindows(mousePosition);
	
				} else 
					positionControls.HideControls();

			} else
				positionControls.HideControls();

			// check your kids next
			foreach ( Control c in Controls )
				if ( c.GetType() != typeof(Splitter) )
					hasMouse = ( c as PaneBase ).UpdateDragWindow(mousePosition, hasMouse);

			return tryMouse;
		}

		public override void Close() {

			base.Close();

			Interlocked.Exchange(ref removing, 1);

			while ( Controls.Count > 0 ) {
				Control c = Controls[0];
				if ( c.GetType().BaseType == typeof(PaneBase) )
					( c as PaneBase ).Close();
				c.Dispose();
				c.Parent = null;
			}

			
			this.Parent = null;
			this.Dispose();

			Interlocked.Exchange(ref removing, 0);
		}
	}

}
