/* 
 * Copyright (c)2007-2008, Dustin Spicuzza
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of the Dustin Spicuzza may not be used to endorse or promote 
 *       products derived from this software without specific prior written 
 *       permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Dustin Spicuzza ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Dustin Spicuzza BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace Panes {

	/// <summary>
	/// Shown at the four sides of the root PaneContainer, and internally by the PaneContainer on top.
	/// </summary>
	internal partial class PositionForm : Form {

		Size sz;

		private PositionForm() {
		}

		public PositionForm(DockStyle position) {

			InitializeComponent();

			// we do this because of there are issues for some reason
			sz = Size;
			this.Position = position;

			this.Load += new EventHandler(PositionForm_Load);
		}

		protected override void OnCreateControl() {
			base.OnCreateControl();
			this.Visible = false;
			this.BackColor = SystemColors.ControlDark;
		}

		void PositionForm_Load(object sender, EventArgs e) {
			Size = sz;
		}

		private DockStyle position;

		internal DockStyle Position {
			get { return position; }
			set { 
				position = value;

				Bitmap b = new Bitmap(Width,Height);
				Graphics g = Graphics.FromImage(b);

				RectangleF centerRect = new RectangleF(Width / 8, Height / 8, ( Width * 6 ) / 8, ( Height * 6 ) / 8);

				// draw the window part
				g.FillRectangle(SystemBrushes.Control, centerRect);

				// draw the window part and the triangle
				PointF []  triPoints = new PointF[3]; ;

	
				// draw a window and an arrow
				switch ( position ) {
					case DockStyle.Top:

						g.FillRectangle(SystemBrushes.Window, centerRect.X, centerRect.Y + centerRect.Height/2, centerRect.Width, centerRect.Height / 2);

						triPoints[0] = new PointF(centerRect.X + centerRect.Width / 2, centerRect.Y + ( centerRect.Height * 5 ) / 8);
						triPoints[1] = new PointF(centerRect.X + centerRect.Width / 4, centerRect.Y + ( centerRect.Height * 7 ) / 8);
						triPoints[2] = new PointF(centerRect.X + ( centerRect.Width * 3 ) / 4, centerRect.Y + ( centerRect.Height * 7 ) / 8);

						break;
					case DockStyle.Bottom:

						g.FillRectangle(SystemBrushes.Window, centerRect.X, centerRect.Y, centerRect.Width, centerRect.Height / 2);

						triPoints[0] = new PointF(centerRect.X + centerRect.Width / 2, centerRect.Y + ( centerRect.Height * 3 ) / 8);
						triPoints[1] = new PointF(centerRect.X + centerRect.Width / 4, centerRect.Y + ( centerRect.Height * 1 ) / 8);
						triPoints[2] = new PointF(centerRect.X + ( centerRect.Width * 3 ) / 4, centerRect.Y + ( centerRect.Height * 1 ) / 8);

						break;
					case DockStyle.Left:

						g.FillRectangle(SystemBrushes.Window, centerRect.X + centerRect.Width / 2, centerRect.Y, centerRect.Width / 2, centerRect.Height);

						triPoints[0] = new PointF(centerRect.X + ( centerRect.Width * 5 ) / 8, centerRect.Y + centerRect.Height / 2);
						triPoints[1] = new PointF(centerRect.X + ( centerRect.Width * 7 ) / 8, centerRect.Y + ( centerRect.Height ) / 4);
						triPoints[2] = new PointF(centerRect.X + ( centerRect.Width * 7 ) / 8, centerRect.Y + ( centerRect.Height * 3) / 4);

						break;
					case DockStyle.Right:

						g.FillRectangle(SystemBrushes.Window, centerRect.X + centerRect.Width / 2, centerRect.Y, centerRect.Width / 2, centerRect.Height);

						triPoints[0] = new PointF(centerRect.X + ( centerRect.Width * 3 ) / 8, centerRect.Y + centerRect.Height / 2);
						triPoints[1] = new PointF(centerRect.X + ( centerRect.Width ) / 8, centerRect.Y + ( centerRect.Height ) / 4);
						triPoints[2] = new PointF(centerRect.X + ( centerRect.Width ) / 8, centerRect.Y + ( centerRect.Height * 3 ) / 4);

						break;
				}

				g.FillPolygon(SystemBrushes.ControlDark, triPoints);				
				
				// draw border
				g.DrawRectangle(SystemPens.WindowFrame, centerRect.X, centerRect.Y, centerRect.Width, centerRect.Height);

				g.Dispose();

				this.BackgroundImage = b;
			}
		}

		private bool isSelected = false;

		/// <summary>
		/// Read this to determine whether this was the selected control
		/// </summary>
		public bool IsSelected {
			get { return isSelected; }
			set {

				if (isSelected == value)
					return;

				if ( !value ) {
					if ( trackTimer.Enabled )
						trackTimer.Stop();

					this.BackColor = SystemColors.ControlDark;

				} else
					this.BackColor = SystemColors.HotTrack;
			
				isSelected = value;
			}
		}


		public void UpdateDragWindow(Point mousePosition) {

			if ( this.RectangleToScreen(ClientRectangle).Contains(mousePosition) ) {

				IsSelected = true;

				if ( MouseButtons == MouseButtons.None ) {
					trackTimer.Stop();
					PaneBase.PaneMoving = false;
				} else if ( !trackTimer.Enabled ) {
					trackTimer.Start();
				}

			} else {

				IsSelected = false;
				
				if ( trackTimer.Enabled )
					trackTimer.Stop();
			}
		}

		private void trackTimer_Tick(object sender, EventArgs e) {
			UpdateDragWindow(new Point(MousePosition.X,MousePosition.Y));
		}
	}
}